ALTER TABLE tbl_jambo_roll_detail
ADD str_roll_code varchar(50) null
GO

ALTER TABLE tbl_jambo_roll_detail
ADD dec_gross_wt decimal(15,3) null
GO

ALTER TABLE tbl_jambo_roll_detail
DROP COLUMN str_product_code
GO


Alter view vw_jambo_roll_detail
as
select
	tbl_jambo_roll_detail.int_unique_id,
	tbl_jambo_roll_detail.int_jambo_roll_id,
	tbl_jambo_roll_detail.dec_length,
	tbl_jambo_roll_detail.dec_width,
	tbl_jambo_roll.str_jambo_roll_no,
	tbl_jambo_roll.str_date_code,
	tbl_jambo_roll.str_product_code,
	vw_jambo_roll.dt_production,
	vw_jambo_roll.int_leather_color_id,
	vw_jambo_roll.str_leather_color_code,
	vw_jambo_roll.str_leather_color_name,	
	tbl_jambo_roll_detail.str_grain,
	tbl_jambo_roll_detail.str_backer,
	tbl_jambo_roll_detail.dec_gauge,
	tbl_jambo_roll_detail.str_grade,
	tbl_jambo_roll_detail.str_roll_code,
	tbl_jambo_roll_detail.dec_gross_wt
	
from
	tbl_jambo_roll_detail
	left join tbl_jambo_roll on (tbl_jambo_roll_detail.int_jambo_roll_id = tbl_jambo_roll.int_jambo_roll_id)	
	left join vw_jambo_roll on (tbl_jambo_roll_detail.int_jambo_roll_id = vw_jambo_roll.int_jambo_roll_id)
GO

GO
/*
	Name:	sp_jambo_roll_detail_insert
	Date:	05/03/2017 12:09:56 PM
*/
alter PROCEDURE sp_jambo_roll_detail_insert
	@int_jambo_roll_id int,
	@int_fl_packing_slip_id int,
	@dec_width decimal(15,2),
	@str_grain varchar(50),
	@str_backer varchar(50),
	@dec_gauge decimal(15,2),
	@str_grade varchar(50),
	@dec_length decimal(15,2),
	@intReturn int = 0 OUTPUT
As
BEGIN
	begin transaction
	Declare @intNextID INT
	-- Get Next ID And No
	--Select @intNextID = ISNULL(MAX(int_jambo_roll_detail_id), 0) + 1 FROM tbl_jambo_roll_detail
	Select @intNextID =  DATEDIFF(s,'2017-01-01',GETDATE())
	If @intNextID = 0
	BEGIN
		rollback transaction
		Return @intReturn
	End
 
	INSERT INTO tbl_jambo_roll_detail (
		int_unique_id, int_jambo_roll_id, int_fl_packing_slip_id, dec_width, str_grain, str_backer, dec_gauge, str_grade, dec_length
	) VALUES (
		@intNextID, @int_jambo_roll_id, @int_fl_packing_slip_id, @dec_width, @str_grain, @str_backer, @dec_gauge, @str_grade, @dec_length
	)
 
	If @@Error <> 0
	BEGIN
		rollback transaction
		Return @intReturn
	End
	commit transaction
	Set @intReturn = @intNextID
	Return @intReturn
End
GO


GO
/*
	Name:	sp_jambo_roll_detail_insert
	Date:	16/03/2017 10:43:37 AM
*/
CREATE PROCEDURE sp_jambo_roll_detail_insert
	@int_jambo_roll_id int,
	@int_fl_packing_slip_id int,
	@dec_width decimal,
	@str_grain varchar(50),
	@str_backer varchar(50),
	@str_grade varchar(50),
	@dec_length decimal,
	@dec_gauge decimal,
	@dec_gross_wt decimal,
	@str_roll_code varchar(50),
	@intReturn int = 0 OUTPUT
As
BEGIN
	begin transaction
	Declare @intNextID INT
	-- Get Next ID And No
	--Select @intNextID = ISNULL(MAX(int_jambo_roll_detail_id), 0) + 1 FROM tbl_jambo_roll_detail
	Select @intNextID =  DATEDIFF(s,'2017-01-01',GETDATE())
	If @intNextID = 0
	BEGIN
		rollback transaction
		Return @intReturn
	End
 
	INSERT INTO tbl_jambo_roll_detail (
		int_unique_id, int_jambo_roll_id, int_fl_packing_slip_id, dec_width, str_grain, str_backer, str_grade, dec_length, dec_gauge, dec_gross_wt, str_roll_code
	) VALUES (
		@intNextID, @int_jambo_roll_id, @int_fl_packing_slip_id, @dec_width, @str_grain, @str_backer, @str_grade, @dec_length, @dec_gauge, @dec_gross_wt, @str_roll_code
	)
 
	If @@Error <> 0
	BEGIN
		rollback transaction
		Return @intReturn
	End
	commit transaction
	Set @intReturn = @intNextID
	Return @intReturn
End
GO
GO
/*
	Name:	sp_jambo_roll_detail_update
	Date:	16/03/2017 10:43:37 AM
*/
CREATE PROCEDURE sp_jambo_roll_detail_update
	@int_unique_id bigint,
	@int_jambo_roll_id int,
	@int_fl_packing_slip_id int,
	@dec_width decimal,
	@str_grain varchar(50),
	@str_backer varchar(50),
	@str_grade varchar(50),
	@dec_length decimal,
	@dec_gauge decimal,
	@dec_gross_wt decimal,
	@str_roll_code varchar(50)
As
BEGIN
	begin transaction
	UPDATE tbl_jambo_roll_detail Set
		int_jambo_roll_id = @int_jambo_roll_id, 
		int_fl_packing_slip_id = @int_fl_packing_slip_id, 
		dec_width = @dec_width, 
		str_grain = @str_grain, 
		str_backer = @str_backer, 
		str_grade = @str_grade, 
		dec_length = @dec_length, 
		dec_gauge = @dec_gauge, 
		dec_gross_wt = @dec_gross_wt, 
		str_roll_code = @str_roll_code

	WHERE int_jambo_roll_detail_id = @int_jambo_roll_detail_id
	If @@Error <> 0
	begin
		rollback transaction
		Return
	End 
	commit transaction
End
GO
 
GO
/*
	Name:	sp_jambo_roll_detail_delete
	Date:	16/03/2017 10:43:37 AM
*/
CREATE PROCEDURE sp_jambo_roll_detail_delete
	@int_unique_id bigint
AS
BEGIN
	Declare @intCount  INT	Select @intCount = count(*) from tbl_fl_packing_slip_detail where int_unique_id= @int_unique_id
	If @intCount > 0 
	begin
		raiserror ('Item in Use:tbl_jambo_roll_detail / tbl_fl_packing_slip_detail / This Item Is being reference In other items',16,1)
		return
	End


	DELETE From tbl_jambo_roll_detail Where int_unique_id = @int_unique_id
End
GO
